<?php
date_default_timezone_set('Asia/Chongqing');
/**
 * 新浪微博操作类
 *
 * @package sae
 * @author Easy Chen
 * @version 1.0
 */
class WeiboClient
{
    /**
     * 构造函数
     *
     * @access public
     * @param mixed $akey 微博开放平台应用APP KEY
     * @param mixed $skey 微博开放平台应用APP SECRET
     * @param mixed $accecss_token OAuth认证返回的token
     * @param mixed $accecss_token_secret OAuth认证返回的token secret
     * @return void
     */
    function __construct( $akey , $skey , $accecss_token , $accecss_token_secret )
    {
        $this->oauth = new WeiboOAuth( $akey , $skey , $accecss_token , $accecss_token_secret );
    }

    /**
     * 最新公共微博
     *
     * @access public
     * @return array
     */
    function public_timeline()
    {
        return $this->oauth->get('http://api.t.sina.com.cn/statuses/public_timeline.json');
    }

    /**
     * 最新关注人微博
     *
     * @access public
     * @return array
     */
    function friends_timeline()
    {
        return $this->home_timeline();
    }

    /**
     * 最新关注人微博
     *
     * @access public
     * @return array
     */
    function home_timeline()
    {
        return $this->oauth->get('http://api.t.sina.com.cn/statuses/home_timeline.json');
    }

    /**
     * 最新 @用户的
     *
     * @access public
     * @param int $page 返回结果的页序号。
     * @param int $count 每次返回的最大记录数（即页面大小），不大于200，默认为20。
     * @return array
     */
    function mentions( $page = 1 , $count = 20 )
    {
        return $this->request_with_pager( 'http://api.t.sina.com.cn/statuses/mentions.json' , $page , $count );
    }


    /**
     * 发表微博
     *
     * @access public
     * @param mixed $text 要更新的微博信息。
     * @return array
     */
    function update( $text )
    {
        //  http://api.t.sina.com.cn/statuses/update.json
        $param = array();
        $param['status'] = $text;

        return $this->oauth->post( 'http://api.t.sina.com.cn/statuses/update.json' , $param );
    }

    /**
     * 发表图片微博
     *
     * @access public
     * @param string $text 要更新的微博信息。
     * @param string $text 要发布的图片路径,支持url。[只支持png/jpg/gif三种格式,增加格式请修改get_image_mime方法]
     * @return array
     */
    function upload( $text , $pic_path )
    {
        //  http://api.t.sina.com.cn/statuses/update.json
        $param = array();
        $param['status'] = $text;
        $param['pic'] = '@'.$pic_path;

        return $this->oauth->post( 'http://api.t.sina.com.cn/statuses/upload.json' , $param , true );
    }

    /**
     * 获取单条微博
     *
     * @access public
     * @param mixed $sid 要获取已发表的微博ID
     * @return array
     */
    function show_status( $sid )
    {
        return $this->oauth->get( 'http://api.t.sina.com.cn/statuses/show/' . $sid . '.json' );
    }

    /**
     * 删除微博
     *
     * @access public
     * @param mixed $sid 要删除的微博ID
     * @return array
     */
    function delete( $sid )
    {
        return $this->destroy( $sid );
    }

    /**
     * 删除微博
     *
     * @access public
     * @param mixed $sid 要删除的微博ID
     * @return array
     */
    function destroy( $sid )
    {
        return $this->oauth->post( 'http://api.t.sina.com.cn/statuses/destroy/' . $sid . '.json' );
    }

    /**
     * 个人资料
     *
     * @access public
     * @param mixed $uid_or_name 用户UID或微博昵称。
     * @return array
     */
    function show_user( $uid_or_name = null )
    {
        return $this->request_with_uid( 'http://api.t.sina.com.cn/users/show.json' ,  $uid_or_name );
    }

    /**
     * 关注人列表
     *
     * @access public
     * @param bool $cursor 单页只能包含100个关注列表，为了获取更多则cursor默认从-1开始，通过增加或减少cursor来获取更多的关注列表
     * @param bool $count 每次返回的最大记录数（即页面大小），不大于200,默认返回20
     * @param mixed $uid_or_name 要获取的 UID或微博昵称
     * @return array
     */
    function friends( $cursor = false , $count = false , $uid_or_name = null )
    {
        return $this->request_with_uid( 'http://api.t.sina.com.cn/statuses/friends.json' ,  $uid_or_name , false , $count , $cursor );
    }

    /**
     * 粉丝列表
     *
     * @access public
     * @param bool $cursor 单页只能包含100个粉丝列表，为了获取更多则cursor默认从-1开始，通过增加或减少cursor来获取更多的粉丝列表
     * @param bool $count 每次返回的最大记录数（即页面大小），不大于200,默认返回20。
     * @param mixed $uid_or_name  要获取的 UID或微博昵称
     * @return array
     */
    function followers( $cursor = false , $count = false , $uid_or_name = null )
    {
        return $this->request_with_uid( 'http://api.t.sina.com.cn/statuses/followers.json' ,  $uid_or_name , false , $count , $cursor );
    }

    /**
     * 关注一个用户
     *
     * @access public
     * @param mixed $uid_or_name 要关注的用户UID或微博昵称
     * @return array
     */
    function follow( $uid_or_name )
    {
        return $this->request_with_uid( 'http://api.t.sina.com.cn/friendships/create.json' ,  $uid_or_name ,  false , false , false , true  );
    }

    /**
     * 取消关注某用户
     *
     * @access public
     * @param mixed $uid_or_name 要取消关注的用户UID或微博昵称
     * @return array
     */
    function unfollow( $uid_or_name )
    {
        return $this->request_with_uid( 'http://api.t.sina.com.cn/friendships/destroy.json' ,  $uid_or_name ,  false , false , false , true);
    }

    /**
     * 返回两个用户关系的详细情况
     *
     * @access public
     * @param mixed $uid_or_name 要判断的用户UID
     * @return array
     */
    function is_followed( $uid_or_name )
    {
        $param = array();
        if( is_numeric( $uid_or_name ) ) $param['target_id'] = $uid_or_name;
        else $param['target_screen_name'] = $uid_or_name;

        return $this->oauth->get( 'http://api.t.sina.com.cn/friendships/show.json' , $param );
    }

    /**
     * 用户发表微博列表
     *
     * @access public
     * @param int $page 页码
     * @param int $count 每次返回的最大记录数，最多返回200条，默认20。
     * @param mixed $uid_or_name 指定用户UID或微博昵称
     * @return array
     */
    function user_timeline( $page = 1 , $count = 20 , $uid_or_name = null )
    {
        if( !is_numeric( $page ) )
            return $this->request_with_uid( 'http://api.t.sina.com.cn/statuses/user_timeline.json' ,  $page );
        else
            return $this->request_with_uid( 'http://api.t.sina.com.cn/statuses/user_timeline.json' ,  $uid_or_name , $page , $count );
    }

    /**
     * 获取私信列表
     *
     * @access public
     * @param int $page 页码
     * @param int $count 每次返回的最大记录数，最多返回200条，默认20。
     * @return array
     */
    function list_dm( $page = 1 , $count = 20  )
    {
        return $this->request_with_pager( 'http://api.t.sina.com.cn/direct_messages.json' , $page , $count );
    }

    /**
     * 发送的私信列表
     *
     * @access public
     * @param int $page 页码
     * @param int $count 每次返回的最大记录数，最多返回200条，默认20。
     * @return array
     */
    function list_dm_sent( $page = 1 , $count = 20 )
    {
        return $this->request_with_pager( 'http://api.t.sina.com.cn/direct_messages/sent.json' , $page , $count );
    }

    /**
     * 发送私信
     *
     * @access public
     * @param mixed $uid_or_name UID或微博昵称
     * @param mixed $text 要发生的消息内容，文本大小必须小于300个汉字。
     * @return array
     */
    function send_dm( $uid_or_name , $text )
    {
        $param = array();
        $param['text'] = $text;

        if( is_numeric( $uid_or_name ) ) $param['user_id'] = $uid_or_name;
        else $param['screen_name'] = $uid_or_name;

        return $this->oauth->post( 'http://api.t.sina.com.cn/direct_messages/new.json' , $param  );
    }

    /**
     * 删除一条私信
     *
     * @access public
     * @param mixed $did 要删除的私信主键ID
     * @return array
     */
    function delete_dm( $did )
    {
        return $this->oauth->post( 'http://api.t.sina.com.cn/direct_messages/destroy/' . $did . '.json' );
    }

    /**
     * 转发一条微博信息。
     *
     * @access public
     * @param mixed $sid 转发的微博ID
     * @param bool $text 添加的转发信息。
     * @return array
     */
    function repost( $sid , $text = false )
    {
        $param = array();
        $param['id'] = $sid;
        if( $text ) $param['status'] = $text;

        return $this->oauth->post( 'http://api.t.sina.com.cn/statuses/repost.json' , $param  );
    }

    /**
     * 对一条微博信息进行评论
     *
     * @access public
     * @param mixed $sid 要评论的微博id
     * @param mixed $text 评论内容
     * @param bool $cid 要评论的评论id
     * @return array
     */
    function send_comment( $sid , $text , $cid = false )
    {
        $param = array();
        $param['id'] = $sid;
        $param['comment'] = $text;
        if( $cid ) $param['cid '] = $cid;

        return $this->oauth->post( 'http://api.t.sina.com.cn/statuses/comment.json' , $param  );

    }

    /**
     * 发出的评论
     *
     * @access public
     * @param int $page 页码
     * @param int $count 每次返回的最大记录数，最多返回200条，默认20。
     * @return array
     */
    function comments_by_me( $page = 1 , $count = 20 )
    {
        return $this->request_with_pager( 'http://api.t.sina.com.cn/statuses/comments_by_me.json' , $page , $count );
    }

    /**
     * 最新评论(按时间)
     *
     * @access public
     * @param int $page 页码
     * @param int $count 每次返回的最大记录数，最多返回200条，默认20。
     * @return array
     */
    function comments_timeline( $page = 1 , $count = 20 )
    {
        return $this->request_with_pager( 'http://api.t.sina.com.cn/statuses/comments_timeline.json' , $page , $count );
    }

    /**
     * 单条评论列表(按微博)
     *
     * @access public
     * @param mixed $sid 指定的微博ID
     * @param int $page 页码
     * @param int $count 每次返回的最大记录数，最多返回200条，默认20。
     * @return array
     */
    function get_comments_by_sid( $sid , $page = 1 , $count = 20 )
    {
        $param = array();
        $param['id'] = $sid;
        if( $page ) $param['page'] = $page;
        if( $count ) $param['count'] = $count;

        return $this->oauth->get('http://api.t.sina.com.cn/statuses/comments.json' , $param );

    }

    /**
     * 批量统计微博的评论数，转发数，一次请求最多获取100个。
     *
     * @access public
     * @param mixed $sids 微博ID号列表，用逗号隔开
     * @return array
     */
    function get_count_info_by_ids( $sids )
    {
        $param = array();
        $param['ids'] = $sids;

        return $this->oauth->get( 'http://api.t.sina.com.cn/statuses/counts.json' , $param );
    }

    /**
     * 对一条微博评论信息进行回复。
     *
     * @access public
     * @param mixed $sid 微博id
     * @param mixed $text 评论内容。
     * @param mixed $cid 评论id
     * @return array
     */
    function reply( $sid , $text , $cid )
    {
        $param = array();
        $param['id'] = $sid;
        $param['comment'] = $text;
        $param['cid '] = $cid;

        return $this->oauth->post( 'http://api.t.sina.com.cn/statuses/reply.json' , $param  );

    }

    /**
     * 返回用户的发布的最近20条收藏信息，和用户收藏页面返回内容是一致的。
     *
     * @access public
     * @param bool $page 返回结果的页序号。
     * @return array
     */
    function get_favorites( $page = false )
    {
        $param = array();
        if( $page ) $param['page'] = $page;

        return $this->oauth->get( 'http://api.t.sina.com.cn/favorites.json' , $param );
    }

    /**
     * 收藏一条微博信息
     *
     * @access public
     * @param mixed $sid 收藏的微博id
     * @return array
     */
    function add_to_favorites( $sid )
    {
        $param = array();
        $param['id'] = $sid;

        return $this->oauth->post( 'http://api.t.sina.com.cn/favorites/create.json' , $param );
    }

    /**
     * 删除微博收藏。
     *
     * @access public
     * @param mixed $sid 要删除的收藏微博信息ID.
     * @return array
     */
    function remove_from_favorites( $sid )
    {
        return $this->oauth->post( 'http://api.t.sina.com.cn/favorites/destroy/' . $sid . '.json'  );
    }


    function verify_credentials()
    {
        return $this->oauth->get( 'http://api.t.sina.com.cn/account/verify_credentials.json' );
    }

    function update_avatar( $pic_path )
	{
		$param = array();
		$param['image'] = "@".$pic_path;

        return $this->oauth->post( 'http://api.t.sina.com.cn/account/update_profile_image.json' , $param , true );

	}



    // =========================================

    /**
     * @ignore
     */
    protected function request_with_pager( $url , $page = false , $count = false )
    {
        $param = array();
        if( $page ) $param['page'] = $page;
        if( $count ) $param['count'] = $count;

        return $this->oauth->get($url , $param );
    }

    /**
     * @ignore
     */
    protected function request_with_uid( $url , $uid_or_name , $page = false , $count = false , $cursor = false , $post = false )
    {
        $param = array();
        if( $page ) $param['page'] = $page;
        if( $count ) $param['count'] = $count;
        if( $cursor )$param['cursor'] =  $cursor;

        if( $post ) $method = 'post';
        else $method = 'get';

        if( is_numeric( $uid_or_name ) )
        {
            $param['user_id'] = $uid_or_name;
            return $this->oauth->$method($url , $param );

        }elseif( $uid_or_name !== null )
        {
            $param['screen_name'] = $uid_or_name;
            return $this->oauth->$method($url , $param );
        }
        else
        {
            return $this->oauth->$method($url , $param );
        }

    }

}

/**
 * 新浪微博 OAuth 认证类
 *
 * @package sae
 * @author Easy Chen
 * @version 1.0
 */
class WeiboOAuth {
    /**
     * Contains the last HTTP status code returned.
     *
     * @ignore
     */
    public $http_code;
    /**
     * Contains the last API call.
     *
     * @ignore
     */
    public $url;
    /**
     * Set up the API root URL.
     *
     * @ignore
     */
    public $host = "http://api.t.sina.com.cn/";
    /**
     * Set timeout default.
     *
     * @ignore
     */
    public $timeout = 30;
    /**
     * Set connect timeout.
     *
     * @ignore
     */
    public $connecttimeout = 30;
    /**
     * Verify SSL Cert.
     *
     * @ignore
     */
    public $ssl_verifypeer = FALSE;
    /**
     * Respons format.
     *
     * @ignore
     */
    public $format = 'json';
    /**
     * Decode returned json data.
     *
     * @ignore
     */
    public $decode_json = TRUE;
    /**
     * Contains the last HTTP headers returned.
     *
     * @ignore
     */
    public $http_info;
    /**
     * Set the useragnet.
     *
     * @ignore
     */
    public $useragent = 'Sae T OAuth v0.2.0-beta2';
    /* Immediately retry the API call if the response was not successful. */
    //public $retry = TRUE;




    /**
     * Set API URLS
     */
    /**
     * @ignore
     */
    function accessTokenURL()  { return 'http://api.t.sina.com.cn/oauth/access_token'; }
    /**
     * @ignore
     */
    function authenticateURL() { return 'http://api.t.sina.com.cn/oauth/authenticate'; }
    /**
     * @ignore
     */
    function authorizeURL()    { return 'http://api.t.sina.com.cn/oauth/authorize'; }
    /**
     * @ignore
     */
    function requestTokenURL() { return 'http://api.t.sina.com.cn/oauth/request_token'; }


    /**
     * Debug helpers
     */
    /**
     * @ignore
     */
    function lastStatusCode() { return $this->http_status; }
    /**
     * @ignore
     */
    function lastAPICall() { return $this->last_api_call; }

    /**
     * construct WeiboOAuth object
     */
    function __construct($consumer_key, $consumer_secret, $oauth_token = NULL, $oauth_token_secret = NULL) {
        $this->sha1_method = new OAuthSignatureMethod_HMAC_SHA1();
        $this->consumer = new OAuthConsumer($consumer_key, $consumer_secret);
        if (!empty($oauth_token) && !empty($oauth_token_secret)) {
            $this->token = new OAuthConsumer($oauth_token, $oauth_token_secret);
        } else {
            $this->token = NULL;
        }
    }


    /**
     * Get a request_token from Weibo
     *
     * @return array a key/value array containing oauth_token and oauth_token_secret
     */
    function getRequestToken($oauth_callback = NULL) {
        $parameters = array();
        if (!empty($oauth_callback)) {
            $parameters['oauth_callback'] = $oauth_callback;
        }

        $request = $this->oAuthRequest($this->requestTokenURL(), 'GET', $parameters);
        $token = OAuthUtil::parse_parameters($request);
        $this->token = new OAuthConsumer($token['oauth_token'], $token['oauth_token_secret']);
        return $token;
    }

    /**
     * Get the authorize URL
     *
     * @return string
     */
    function getAuthorizeURL($token, $sign_in_with_Weibo = TRUE , $url) {
        if (is_array($token)) {
            $token = $token['oauth_token'];
        }
        if (empty($sign_in_with_Weibo)) {
            return $this->authorizeURL() . "?oauth_token={$token}&oauth_callback=" . urlencode($url);
        } else {
            return $this->authenticateURL() . "?oauth_token={$token}&oauth_callback=". urlencode($url);
        }
    }

    /**
     * Exchange the request token and secret for an access token and
     * secret, to sign API calls.
     *
     * @return array array("oauth_token" => the access token,
     *                "oauth_token_secret" => the access secret)
     */
    function getAccessToken($oauth_verifier = FALSE, $oauth_token = false) {
        $parameters = array();
        if (!empty($oauth_verifier)) {
            $parameters['oauth_verifier'] = $oauth_verifier;
        }


        $request = $this->oAuthRequest($this->accessTokenURL(), 'GET', $parameters);
        $token = OAuthUtil::parse_parameters($request);
        $this->token = new OAuthConsumer($token['oauth_token'], $token['oauth_token_secret']);
        return $token;
    }

    /**
     * GET wrappwer for oAuthRequest.
     *
     * @return mixed
     */
    function get($url, $parameters = array()) {
        $response = $this->oAuthRequest($url, 'GET', $parameters);
        if ($this->format === 'json' && $this->decode_json) {
            return json_decode($response, true);
        }
        return $response;
    }

    /**
     * POST wreapper for oAuthRequest.
     *
     * @return mixed
     */
    function post($url, $parameters = array() , $multi = false) {

        $response = $this->oAuthRequest($url, 'POST', $parameters , $multi );
        if ($this->format === 'json' && $this->decode_json) {
            return json_decode($response, true);
        }
        return $response;
    }

    /**
     * DELTE wrapper for oAuthReqeust.
     *
     * @return mixed
     */
    function delete($url, $parameters = array()) {
        $response = $this->oAuthRequest($url, 'DELETE', $parameters);
        if ($this->format === 'json' && $this->decode_json) {
            return json_decode($response, true);
        }
        return $response;
    }

    /**
     * Format and sign an OAuth / API request
     *
     * @return string
     */
    function oAuthRequest($url, $method, $parameters , $multi = false) {

        if (strrpos($url, 'http://') !== 0 && strrpos($url, 'http://') !== 0) {
            $url = "{$this->host}{$url}.{$this->format}";
        }

        // echo $url ;
        $request = OAuthRequest::from_consumer_and_token($this->consumer, $this->token, $method, $url, $parameters);
        $request->sign_request($this->sha1_method, $this->consumer, $this->token);
        switch ($method) {
        case 'GET':
            //echo $request->to_url();
            return $this->http($request->to_url(), 'GET');
        default:
            return $this->http($request->get_normalized_http_url(), $method, $request->to_postdata($multi) , $multi );
        }
    }

    /**
     * Make an HTTP request
     *
     * @return string API results
     */
    function http($url, $method, $postfields = NULL , $multi = false) {
        $this->http_info = array();
        $ci = curl_init();
        /* Curl settings */
        curl_setopt($ci, CURLOPT_USERAGENT, $this->useragent);
        curl_setopt($ci, CURLOPT_CONNECTTIMEOUT, $this->connecttimeout);
        curl_setopt($ci, CURLOPT_TIMEOUT, $this->timeout);
        curl_setopt($ci, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ci, CURLOPT_SSL_VERIFYPEER, $this->ssl_verifypeer);
        curl_setopt($ci, CURLOPT_HEADERFUNCTION, array($this, 'getHeader'));
        curl_setopt($ci, CURLOPT_HEADER, FALSE);

        switch ($method) {
            case 'POST':
                curl_setopt($ci, CURLOPT_POST, TRUE);
                if (!empty($postfields)) {
                    curl_setopt($ci, CURLOPT_POSTFIELDS, $postfields);
                }
                break;
            case 'DELETE':
                curl_setopt($ci, CURLOPT_CUSTOMREQUEST, 'DELETE');
                if (!empty($postfields)) {
                    $url = "{$url}?{$postfields}";
                }
        }

        $header_array = array();
        $header_array2=array();
        if( $multi )
        	$header_array2 = array("Content-Type: multipart/form-data; boundary=" . OAuthUtil::$boundary , "Expect: ");
        foreach($header_array as $k => $v)
            array_push($header_array2,$k.': '.$v);

        curl_setopt($ci, CURLOPT_HTTPHEADER, $header_array2 );
        curl_setopt($ci, CURLINFO_HEADER_OUT, TRUE );
        curl_setopt($ci, CURLOPT_URL, $url);

        $response = curl_exec($ci);
        $this->http_code = curl_getinfo($ci, CURLINFO_HTTP_CODE);
        $this->http_info = array_merge($this->http_info, curl_getinfo($ci));
        $this->url = $url;
        curl_close ($ci);
        return $response;
    }

    /**
     * Get the header info to store.
     *
     * @return int
     */
    function getHeader($ch, $header) {
        $i = strpos($header, ':');
        if (!empty($i)) {
            $key = str_replace('-', '_', strtolower(substr($header, 0, $i)));
            $value = trim(substr($header, $i + 2));
            $this->http_header[$key] = $value;
        }
        return strlen($header);
    }
}